/*************************************************************************/
/*!
* \file main.c
*
* \brief 
*
* \par Funktionsweise: programm zum testen der UART kommunication
*	
* \version	V--- - 26.03.2007 - Olaf Petersen\n
*			Created\n
*************************************************************************/

#include <avr/io.h> 	//I/O-Register 
#include <stdint.h>		//standart Integertypen
#include <stdlib.h> 	// itoa
#include <avr/interrupt.h>
#include <util/delay.h>

#include "../nbot_lib/globals.h"
//#include "../nbot_lib/adconvert.h"
#include "../nbot_lib/uart.h"
#include "../nbot_lib/timer.h"
//#include "../nbot_lib/motor.h"
//#include "../nbot_lib/encoder.h"
//#include "../nbot_lib/pcc.h"
#include "../nbot_lib/fifo.h"

#include "../nbot_lib/rf12.h"  // wird noch getestet 
  

#ifndef F_CPU
#define F_CPU 11059000UL
#endif

#define LED1 (1<<PB2)							///< Pin der LED 1
#define LED2 (1<<PB3)							///< Pin der LED 2
#define LED1_AN	PORTB &= ~LED1		///< Schaltet die LED 1 an
#define LED1_AUS	PORTB |= LED1		///< Schaltet die LED 1 aus
#define LED1_TOG	PORTB ^= LED1		///< wechselt den Zustand der LED 1
#define LED2_AN	PORTB &= ~LED2		///< Schaltet die LED 2 an
#define LED2_AUS	PORTB |= LED2		///< Schaltet die LED 2 aus
#define LED2_TOG	PORTB ^= LED2		///< wechselt den Zustand der LED 2


#define ki 2	/*!< \brief Integrationszeitkonstante */
#define kd 0	/*!< \brief Differenzierungszeitkonstante */
#define kp 2	/*!< \brief Verstrkung */


//**** variablen ****
uint32_t zeit = 0;
uint32_t startzeit = 0;
uint8_t *rfm= 0;
/*
int16_t sollwert[] = {0,0};	
int16_t istwert[] = {0,0};
int16_t fehler[] = {0,0};
int16_t fehler_alt[] = {0, 0};
int16_t	integrator[] = {0, 0};
int16_t differenzierer[] = {0, 0};
int16_t stellwert[] = {0, 0};

pcc_t pcc_daten;
*/
unsigned char test[100]="\n";
#define BUF_SIZE 10					/*!< \brief Puffergre fr Lesepuffer [Byte]*/
uint8_t inbuf[BUF_SIZE];		/*!< \brief Speicher fr FIFO Puffer */
fifo_t infifo;							/*!< \brief Fifo struct */


//**** funktionen ****
//void regelung(uint8_t motor);
//**** funktionen ****
void send(void);
void receive(void);
uint8_t rfm12_getc(uint8_t *date);


//*** main ***
int main(void)
{

	uart_init();											// Uart initialisieren
	timer2_init();										// Timer2 parametrisieren, fr Systemzeit und Tastenentprellung
	
	// Funk
	// FIFOs fr Eingabe initialisieren 
	fifo_init (&infifo, inbuf, BUF_SIZE);
	rf12_init();											// Funkmodul initialisieren
	rf12_setfreq(RF12FREQ(433.92));		// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf12_setbandwidth(5, 1, 4);				// 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
	rf12_setbaud(9600);								// 19200 baud
	rf12_setpower(0, 6);							// 1mW Ausgangangsleistung, 120k

	// PC communication
/*	pcc_daten.time = &zeit;						// Pointer der Systemzeit fr PC communikation speichern
	pcc_daten.encoder[0] = istwert;		// Pointer der Encoderwerte fr PC comminication speichern
	pcc_daten.istwert[0] = istwert;		// Pointer des Drehzahlistwertes fr PC communikation speichern
	pcc_daten.sollwert[0] =  sollwert;
	pcc_daten.rfm = rfm;
	pcc_init(&pcc_daten);							// PC communication initialisieren
*/	
	// Status LEDs
	DDRB |= LED1 | LED2;
	LED1_AUS;
	LED2_AUS;
	
	uint8_t buf;
	// globale Interrups aktivieren
	sei();
	
	
	for(;;)
	{

		receive();	// ber das Funkmodul empfangen
		
		if(Gettime() >= (zeit + 1000))		// alle 100ms
		{	
			zeit = Gettime();								// ruft die aktuelle systemzeit ab	
			/******  Eingnge Lesen *********/
			
			//batteryVoltage = ReadChannel(PA3);		// Liest die BatterieSpannung
			
			// *************  Motorregelung ***************
			//regelung(LEFT);			// linken Motor regeln
			//regelung(RIGHT);		// rechten Motor regeln	
			
			// *************  Serielle Kommunikation ******
			if(rfm12_getc(&buf) == 1)
			{	//pcc_rx(buf);		// PC Kommunikation
			}
			//pcc();
			
			LED2_TOG;
		
		} // end every 100ms
		
	}// end endlesloop
	
	
	return 0;
}




/**************************************************************************/
/*!
* \brief Regelung der Motoren.
*
*	\par Funktionsweise:
*	die Funktion liest die Drehzahlwerte ein und erzeugt daraus einen neuen 
* Motorstellwert.
*
* \param [in]
*		motor	Motor auswahl: LEFT, RIGHT
*
***************************************************************************/
/*void regelung(uint8_t motor)
{
			// alle werte zurcksetzten wenn Motor still stehen soll
			if(sollwert[motor] == 0){
				fehler[motor]=0;
				fehler_alt[motor]=0;
				integrator[motor]=0;
				differenzierer[motor]=0;
				stellwert[motor]=0;
				motor_speed(0, 0);			
				return;
			}
			
			istwert[motor] = encoderGet(motor);
			
			
			//****** linke Regelung ************************************************ 
			
			fehler[motor] = sollwert[motor] - istwert[motor];						// Sollwertabweichung
			
			integrator[motor] += fehler[motor];												// Integrator
			// Integrator begrenzen
			if(integrator[motor] > 1023)
				integrator[motor] = 1023;
			if(integrator[motor] < (-1023))
				integrator[motor] = (-1023);			
			
			differenzierer[motor] = fehler[motor] - fehler_alt[motor];	// Differenzierer
			
			stellwert[motor]  = integrator[motor]/ki;									// Stellgre
			stellwert[motor] += kd * differenzierer[motor];
			stellwert[motor] += kp * fehler[motor];
			
			// Stellwertbegrenzung
			if(stellwert[motor]>255)
				stellwert[motor] = 255;
			if(stellwert[motor]< (-255))
				stellwert[motor] = (-255);
				
			fehler_alt[motor] = fehler[motor];													// Sollwertabweichung merken
		
			switch (motor)
			{
				case LEFT:
					// Motordrehrichtung
					if(stellwert[motor]>0)
						motor_dir(FWD,FREE);
					if(stellwert[motor]<0){
						stellwert[motor] = -stellwert[motor];
						motor_dir(RWD,FREE);
					}
					break;
				case RIGHT:
					// Motordrehrichtung
					if(stellwert[motor]>0)
						motor_dir(FREE,FWD);
					if(stellwert[motor]<0){
						stellwert[motor] = -stellwert[motor];
						motor_dir(FREE,RWD);
					}
					break;
			}
			
			// Motorspeed setzen 
			motor_speed(stellwert[LEFT], stellwert[RIGHT]);
}
*/

// ber funk empfangen
void receive(void)
{	
	uint8_t tmp_sreg;
	tmp_sreg = SREG;
	//cli();
	
	// Empfangen
	if(rf12_rxdata(test))
	{

		LED1_TOG;
		
		// ber Uart ausgeben
		uart_puts((char*)test);
		
		// Pause
		for(long a = 0;a<10000;a++){asm("nop");}
		
		// zurcksenden
		rf12_txdata(test,100);
		
		fifo_put(&infifo, test[0]);
		
		LED1_TOG;
	}

	//SREG = tmp_sreg;

}

// empfangene daten ausgeben
uint8_t rfm12_getc(uint8_t *date)
{
	uint8_t data = 0;
	
	if (fifo_get(&infifo, &data)==0){
		*date = data;
		return 1;
	} else {
		return 0;
	}
}

// ber funk senden
void send(void)
{	
	unsigned char test[]="Dies ist ein 433MHz Test !!!\n  ";	
	rf12_txdata(test,sizeof(test));

}

